package one.shandan;

import java.lang.String;
import java.lang.Boolean;
import java.util.Iterator;
import java.util.List;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Arrays;
import org.crosswire.jsword.passage.VerseRangeFactory;
import org.crosswire.jsword.versification.system.SystemDefault;
import org.crosswire.jsword.versification.BibleBook;
import org.crosswire.jsword.passage.NoSuchVerseException;
import org.crosswire.jsword.versification.Versification;
import org.crosswire.jsword.versification.system.Versifications;
import org.crosswire.jsword.book.Books;
import org.crosswire.jsword.book.Book;
import org.crosswire.jsword.book.BookException;
import org.crosswire.jsword.passage.Key;
import org.crosswire.jsword.passage.NoSuchKeyException;
import org.jdom2.Attribute;
import org.jdom2.Element;
import org.jdom2.Text;
import org.jdom2.Content;
import org.jdom2.Content.CType;
import org.postgresql.pljava.annotation.Function;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import static org.postgresql.pljava.annotation.Function.Effects.IMMUTABLE;
import static org.postgresql.pljava.annotation.Function.OnNullInput.RETURNS_NULL;
import static org.postgresql.pljava.annotation.Function.Trust.SANDBOXED;

public class PlJavaJSword {
  private static final String DEFAULT_BOOK = "MKJV";
  private static final String DEFAULT_VERSIFICATION = "KJV";
  private static final Versification kjv = Versifications.instance().getVersification(DEFAULT_VERSIFICATION);
  private static final Books BOOKS = Books.installed();
  private static final HashSet<Character> PUNCTUATION = new HashSet<Character>(Arrays.asList('.', ',', ';', ':', '?', '!'));
  private static final HashSet<Character> OPEN_PARENTHESES = new HashSet<Character>(Arrays.asList('[', '('));
  private static final HashSet<Character> CLOSE_PARENTHESES = new HashSet<Character>(Arrays.asList(']', ')'));
  private static final Logger logger = LoggerFactory.getLogger(PlJavaJSword.class);

  @Function(onNullInput=RETURNS_NULL, effects=IMMUTABLE, trust=SANDBOXED)
  public static Boolean isValidVerse(String reference) {
    try {
      VerseRangeFactory.fromString(kjv, reference);
      return true;
    } catch (NoSuchVerseException ex) {
    }
    return false;
  }
  @Function(onNullInput=RETURNS_NULL, effects=IMMUTABLE, trust=SANDBOXED)
  public static String normalizeVerse(String reference){
    try {
      return VerseRangeFactory.fromString(kjv, reference).getName();
    } catch (NoSuchVerseException ex) {
      return null;
    }
  }
  private static String join(Iterator<String> parts){
    String ret = "";
    while (parts.hasNext()){
      String part = parts.next().trim();
      if (part.isEmpty()){
          continue;
      }
      if (ret.isEmpty()){
        ret += part;
      }
      else if (OPEN_PARENTHESES.contains(ret.charAt(ret.length()-1))){
        ret += part;
      }
      else if (CLOSE_PARENTHESES.contains(part.charAt(0))){
        ret += part;
      }
      else if (PUNCTUATION.contains(part.charAt(0))){
        ret += part;
      }
      else {
        ret += " " + part;
      }
    }
    return ret;
  }
  private static String parseContent(Iterator<Content> content){
    List<String> ret = new ArrayList<String>();
    while (content.hasNext()){
      Content c = content.next();
      String part = null;
      switch(c.getCType()){
        case Element:
          Element e = (Element)c;
          logger.info(e.getName() + " " + e.getValue());
          switch (e.getName()){
            case "title":
            case "note":
              break;
            default:
              part = parseContent(e.getContent().listIterator());
              ret.add(part);
          }
          break;
        case Text:
          logger.info("Text: " + c.getValue());
          part = c.getValue().trim();
          ret.add(part);
          break;
        default:
          logger.info(c.getCType() + " " + c.getValue());
          break;
      }
    }
    return join(ret.listIterator());
  }
  @Function(onNullInput=RETURNS_NULL, effects=IMMUTABLE, trust=SANDBOXED)
  public static String getText(String translation, String reference) throws BookException {
    Book book = BOOKS.getBook(translation);
    if (book == null){
      return null;
    }
    try {
      Key key = book.getKey(reference);
      return parseContent(book.getOsisIterator(key, false));
    } catch (NoSuchKeyException ex){
      return null;
    }
  }
  @Function(onNullInput=RETURNS_NULL, effects=IMMUTABLE, trust=SANDBOXED)
  public static String getDefaultText(String reference) throws BookException {
    Book book = BOOKS.getBook(DEFAULT_BOOK);
    if (book == null){
      return null;
    }
    try {
      Key key = book.getKey(reference);
      return parseContent(book.getOsisIterator(key, false));
    } catch (NoSuchKeyException ex){
      return null;
    }
  }
  @Function(effects=IMMUTABLE, trust=SANDBOXED)
  public static String getDefaultTranslation(){
    return DEFAULT_BOOK;
  }
}

